/*  Program TempoFs
    Compile with: gcc -g TempoFs.c -o TempoFs -lm  
    Please send bug reports to p.e.jorde@bio.uio.no      

    Copyright (C) 2007 by Per Erik Jorde

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    <http://www.gnu.org/licenses/>
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#define version     "Oct. 22. 2007"
#define MaxAlleles  100   /* maximum number of alleles per locus                */
#define MaxLoci     100   /* maximum number of loci                             */
#define MaxSamples  10    /* maximum number of samples                          */
#define AlleleFreqTolerance 0.01 /* for testing allele frequencies in data file */
#define BLANK       ' '

double MinSize = 1.0;                   /* ignore small samples (MinSize=#genes) */
double x[MaxAlleles+1][MaxSamples+1][MaxLoci+1];    /* observed allele frequency */
double S[MaxSamples+1][MaxLoci+1];                  /* sample sizes (in genes)   */
int K[MaxLoci+1];                                   /* # allelels at each locus  */
int t[MaxSamples+1];                                /* generation number         */
char locusname[MaxLoci+1][256];                     /* names of loci             */
int locus, n_loci, sample, next_sample, n_samples, run_no;            /* various */
int timespan, intervals, sum_k, tot_k, i, Kl, b, rnum;
char ch;
FILE *infile, *outfile;
char *infilename, *outfilename;

/*********************************************************************************/

void OpeningScreen()
{
  printf("                          TempoFs                                 \n\n");
  printf("Estimates variance-effective size (Ne) of a single population from\n");
  printf("observed differences in allele frequencies among temporally spaced\n");
  printf("samples. Up to %i samples scored for up to %i loci (each with\n", MaxSamples, MaxLoci);
  printf("max %i alleles) can be used for input (samples should be ordered).\n\n", MaxAlleles);
  printf("The computed statistic is based on Jorde & Ryman's (2007) estimator\n");
  printf("Fs, computed by summing the nominator and denominator separately  \n");
  printf("over all alleles and loci, before carrying out the division       \n");
  printf("(equation 9):                                                     \n");
  printf("                                                                  \n");
  printf("                       Sum[(x-y)^2]                               \n");
  printf("                  Fs = -------------                              \n");
  printf("                       Sum[z*(1-z)]                               \n");
  printf("where x and y are observed allele frequencies in the 1. and 2.    \n");
  printf("sample, respectively, and z is their mean.                        \n");
  printf("Ne is estimated from the quantity Fs, assuming discrete generation\n");
  printf("intervals (i.e., non-overlapping generations).                    \n");
  printf("For sample plan 1 (cf. equation 14):                              \n");
  printf("                                                                  \n");
  printf("                   T*(1+Fs/4)*(1 - 1/2n_2)                        \n");
  printf("         Ne = ------------------------------------                \n");
  printf("              2*(Fs*(1 - 1/4n + 1/4N) - 1/n + 1/N)                \n");
  printf("where n_2 is the size (diploid individuals) of the last sample, n \n");
  printf("is the harmonic mean size of the two samples, T is the number of  \n");
  printf("generations between the two samples, and N is the actual (census) \n");
  printf("population size.                                                  \n");
  printf("For sample plan 2 (cf. equation 13):                              \n");
  printf("                                                                  \n");
  printf("                    T*(1+Fs/4)*(1 - 1/2n_2)                       \n");
  printf("               Ne = -----------------------                       \n");
  printf("                    2*(Fs*(1 - 1/4n) - 1/n)                       \n");
  printf("Standard errors of estimates are based on jackkife over loci.     \n");
  printf("Written by Per Erik Jorde; last changed %s.\n\n", version);
}  /* end of openingscreen () */


/***************************************************************************/

void Error(char *errorstring) {
  fprintf(stderr,"%s\n", errorstring);
  exit(1);
}  /* Error() */

/***************************************************************************/

void GetData(char *datafilename) {
  /* read allele frequencies and sample size for each allele, locus, and sample
     from an ASCII file of the usual format */
  int allele, sample, locus;
  char errorstr[1024];
  char ch;
  double sumfreq;

  /* initialize */
  for (sample = 1; sample <= MaxSamples; ++sample) {
    t[sample] = 0;
    for (locus  = 1; locus  <= MaxLoci; ++locus) {
      S[sample][locus] = 0;
      K[locus]         = 0;
      for (allele = 1; allele <= MaxAlleles; ++allele)
	x[allele][sample][locus] = 0;
    }
  }
  n_loci = 0;
  n_samples = 0;
  /* read data file */
  infile = fopen(datafilename, "r");
  if (fscanf(infile, "%i%i", &n_samples, &n_loci) < 2)
    Error("Invalid infile format or numeric error"); 
  else if (n_samples > MaxSamples)
    Error("To many samples in this data set");
  else if (n_loci > MaxLoci)
    Error("To many loci in this data set");
  /* skip to next line */
  while (ch != '\n')
    fscanf(infile,"%c", &ch);

  /* get number of alleles at each locus */
  for (locus = 1; locus <= n_loci; locus++) {
    if (fscanf(infile, "%i", &K[locus - 1]) != 1)
      Error("Error in indata file (number of alleles)");
    if (K[locus - 1] > MaxAlleles)
      Error("Too many alleles at one or more loci (adjust MaxAlleles)");
  }
  /* skip to next line */
  while (ch != '\n')
    fscanf(infile,"%c", &ch);

  /* read allele frequencies for one locus at the time */
  for (locus = 1; locus <= n_loci; locus++) {
    for (sample = 1; sample <= n_samples; sample++) {
      sumfreq = 0.0;
      for (allele = 0; allele < K[locus - 1]; allele++) {
	if (fscanf(infile, "%lf", &x[allele][sample - 1][locus - 1]) != 1) {
	  sprintf(errorstr,"Error in indata file: cannot read allele %i at locus %i in sample %i", 
		  allele+1, locus, sample);
	  Error(errorstr);
	}
	sumfreq += x[allele][sample - 1][locus - 1];
      }
      /* check that allele frequency sums to 1.000 (+/- epsilon) */
      if (fabs(sumfreq - 1.000) > AlleleFreqTolerance) {
	sprintf(errorstr,
		"Error in indata file: allele frequencies sum to %f at locus %i in sample %i\n",
		sumfreq, locus, sample),
	  Error(errorstr);
      }
      /* sample size in number of genes (double)  */
      if (fscanf(infile, "%lf", &S[sample - 1][locus - 1]) != 1)
	Error("Error in infilename (sample size)"); 
      do {
	ch = getc(infile);   /* read blanks until locus name   */
	if (ch == '\n')
	  ch = ' ';
      } while (ch == BLANK);
      *locusname[locus - 1] = '\0';
      do {   /* read locus name */
	sprintf(locusname[locus - 1] + strlen(locusname[locus - 1]), "%c", ch);
	ch = getc(infile);
	if (ch == '\n')
	  ch = ' ';
      } while (!((ch == BLANK) | (ch == '\n')));
      /* read generation number         */
      if (fscanf(infile, "%i%*[^\n]", &t[sample - 1]) != 1)
	Error("Error in infilename (generation number)");
    }  /* for sample */
  }  /* for locus */
  fclose(infile);
}  /* GetData() */

/***************************************************************************/

void ComputeF(int L1, int L2, int R1, int R2, double *numerator, double *denominator, 
	      double *n_1, double *n_2, double *tr, int *sum_kl, int skip) {
  /* Computes numerator = Sum[(x-y)^2] and denominator = Sum[z*(1-z)] of Fs 
     over loci L1 to L2 and samples from R1 to R2. 
     'skip' implements jackknife elimination of the skip'th locus, 
     unless skip < 0, in which case all loci are included (the usual procedure.)
     Also returns the number of generations (tr) over which Fs is computed,
     the number of alleles, and the (harmonic mean over alleles) sample sizes. */

  int locus, loci, sample, allele, kl, tl;
  double TEMP, z, n1, n2, n;

  /* initialize */
  *numerator   = 0.0; /* sum of nominator of estimator Fs                               */
  *denominator = 0.0; /* sum of denominator of estimator Fs                             */
  *n_1 = *n_2 = 0.0;  /* harmonic mean (loci) size of the two samples (diploid indivduals) */
  *sum_kl = 0;        /* sum of alleles over loci (numbers actually used)               */
  *tr     = 0.0;      /* timespan in generations between samples                        */
  loci    = 0;        /* number of loci that are unfixed and included  */
  for (locus = L1 - 1; locus < L2; locus++) {     /* compute Fs over locus L1 to L2     */
    if (locus != skip) { /* skip = jackknife elimination of locus */
      tl = 0;
      for (sample = R1; sample <= R2; sample++) {
	if (sample + 1 <= R2) {  /* compute Fs over all samples between samples R1 and R2   */
	  kl = 0;   /* number of alleles at this locus in these two samples */
	  for (allele = 0; allele < K[locus]; allele++) { 
	    /* average allele frequency in the two samples */
	    z = (x[allele][sample - 1][locus] + x[allele][sample][locus])/2.0;
	    /* size of 1. sample (diploid individuals) */
	    n1 = 0.5*S[sample - 1][locus];
	    /* size of 2. sample (diploid individuals) */
	    n2 = 0.5*S[sample][locus];
	    /* harmonic mean number of diploid individuals */
	    n = 2.0/(1.0/n1 + 1.0/n2); 	
	    if (n >= MinSize && z > 0.0 && z < 1.0) {
	      /* allele present in at least one sample and is not fixed in both */
	      TEMP = x[allele][sample - 1][locus] - x[allele][sample][locus];
	      /* Save single-allele values in a data structure for later use */
	      *numerator   += TEMP * TEMP; 
	      *denominator += z*(1.0-z); 
	      /* save harmonic average (over loci) size of the two samples */ 
	      *n_1 += 1.0/n1;
	      *n_2 += 1.0/n2;
	      ++kl; 
	      ++ *sum_kl;
	    }  /* for allele */
	  }
	  tl += t[sample] - t[sample - 1];
	}  /* for sample */
      }
      if (kl > 0) {   /* this locus is included */
	++loci;
	*tr += tl;
      }  /* if */
    } /* if not skip */ 
  }  /* for locus */
  /* return results: */
  if (*sum_kl == 0) /* nothing done */
    return;
  *n_1 = *sum_kl/ *n_1;
  *n_2 = *sum_kl/ *n_2;
  *tr /= loci;   /* average number of generations computed per locus     */
}  /* ComputeF() */


/***************************************************************************/

double ComputeNe(double Fsprim, double tr)
{
  /* computes the Ne corresponding to the estimated Fs' and number of generations.
     Note: Ne < 0 (zero) implies an infinite estimate of effective size   */
  if (Fsprim < -0.0000001 || Fsprim > 0.0000001)
    return (tr / (2 * (Fsprim)));
}  /* ComputeNe() */

int dubsort(const void *p, const void *q) { 
  /*** not used in this version ***/
  double xx;
  const double *pd=p, *qd=q;  /* Convert to pointers to doubles */
  xx = (*pd) - (*qd);
  if (xx > 0.0) return 1;       /* Out of order */
  else if (xx<0.0) return -1;   /* In order */
  /* else a tie value */
  return 0; 
} /* dubsort() */


/********************************Main Program*******************************/
int main(int argc, char *argv[]) {

  double numerator, denominator, n_1, n_2, n_harmonic, Fs, Fsprim, Ne, generations;
  double TEMP, JackFs, JackFsprim, JackFsSE, JackFsprimSE;
  int plan, N, j, resamples;
  
  /* Initialize file pointers etc. */
  outfile = NULL;
  infile  = NULL;
  OpeningScreen();
  /* Open files for reading/writing */
  infilename = malloc(1024);
  outfilename = malloc(1024);
  if (argc > 1) {
    infilename = argv[1];
    printf("Using data from %s\n", infilename);
    outfilename = malloc(256);
    if (argc > 2)
      outfilename = argv[2];
    else { /* No outfile name given: use infile name +.OUT extension */
      sprintf(outfilename,"%s.OUT", infilename);
      printf("Writing resuls to %s\n", outfilename);
    }
  } else {
    /* No filenames given as arguments: ask for them */
    printf("Enter name of data file: ");
    scanf("%s", infilename);
    printf("Enter output file name: ");
    scanf("%s", outfilename);
  } /* if */
  /* Ask for sample plan 1 or 2 */
  printf("Enter sample plan (1 or 2): ");
  if (scanf("%i", &plan) != 1 || plan < 1 || plan > 2)
    Error(" Invalid entry ");
  else if (plan == 1) {
    printf("Enter actual (census) population size, N: ");
    if (scanf("%i", &N) != 1 || N < 0)
      Error(" Invalid entry ");
  }
  outfile = fopen(outfilename, "a");
  if (!outfile)
    Error(" Cannon open outfile for writing ");
  
  /* Read and store data */
  printf("Reading %s\n", infilename);
  GetData(infilename); 
  printf("Processing %s\n", infilename);
  /* Write header for output file */
  fprintf(outfile,"Output from TempoFs (version %s)\n", version);
  fprintf(outfile,"Data are from from file %s\n", infilename);
  fprintf(outfile, 
	  "\nNote: Effective size (Ne = t/2Fs') is estimated under the assumptions that:\n");
  fprintf(outfile,
	  "         - the organism is diploid\n");
  fprintf(outfile,
	  "         - generations are discrete\n");
  fprintf(outfile, 
	  "         - time is measured in generations\n");
  fprintf(outfile,
	  "         - only genetic drift affects allele frequency dynamics\n"); 
  fprintf(outfile,
	  "         - samples are from a single population, drawn according to plan %i\n", plan);
  fprintf(outfile,
	  "         - multiple sample intervals are computed separately\n\n");

  /* Do the calulations for all SUCCESSIVE sample intervals in turn: */

  for (sample = 1; sample < n_samples; sample++) {
    fprintf(outfile, "Estimated drift (Fs') and Ne over the interval from ");
    fprintf(outfile, "%4i to %4i:\n", t[sample - 1], t[sample]);
    fprintf(outfile, 
	    "     Locus     K      S1      S2     Fs           Fs'          t          Ne");
    /* Compute F and Ne locus by locus */
    for (locus = 1; locus <= n_loci; locus++) {
      fprintf(outfile, "\n%10s", locusname[locus - 1]);
      ComputeF(locus, locus, sample, sample + 1, &numerator, &denominator, 
	       &n_1, &n_2, &generations, &Kl,-1);
      n_harmonic = 2.0/(1.0/n_1 + 1.0/n_2); 	
      fprintf(outfile, "%6i  %6.0f  %6.0f  ", Kl, n_1, n_2 /*n_harmonic*/);
      if (Kl > 0) {
	Fs = numerator/denominator;
	if (plan == 2) 	/* use sample correction term for plan II 
			   (equation [13] in Jorde & Ryman 2007) */
	  Fsprim = (Fs*(1.0 - 1.0/(4.0*n_harmonic)) - 1.0/n_harmonic);
	else            /* use sample correction term for plan I 
			   (equation [14] in Jorde & Ryman 2007) */
	  Fsprim = (Fs*(1.0 - 1.0/(4.0*n_harmonic) + 1.0/(4.0*N)) - 1.0/n_harmonic + 1.0/N);
	Fsprim /= ((1.0 + Fs/4.0)*(1-1.0/(2.0*n_2))); /* Note: size of last sample */
	Ne = ComputeNe(Fsprim, generations);
	fprintf(outfile, "%12.7f %12.7f %6.1f  %8.0f", Fs, Fsprim, generations, Ne);
      } else fprintf(outfile,"   fixed            ");
    }  /* for locus */
    fprintf(outfile, "\n  All loci");
    ComputeF(1, n_loci, sample, sample + 1, &numerator, &denominator, 
	     &n_1, &n_2, &generations, &Kl, -1);
    n_harmonic = 2.0/(1.0/n_1 + 1.0/n_2); 	
    fprintf(outfile, "%6i    %6.1f  %6.1f", Kl, n_1, n_2);
    if (Kl > 0) {
      Fs = numerator/denominator;
      if (plan == 2)
	Fsprim = (Fs*(1.0 - 1.0/(4.0*n_harmonic)) - 1.0/n_harmonic);
      else
	Fsprim = (Fs*(1.0 - 1.0/(4.0*n_harmonic) + 1.0/(4.0*N)) - 1.0/n_harmonic + 1.0/N);
      Fsprim /= ((1.0 + Fs/4.0)*(1-1.0/(2.0*n_2)));
      Ne = ComputeNe(Fsprim, generations);
      fprintf(outfile, "%12.7f %12.7f %6.1f  %8.0f", Fs, Fsprim, generations, Ne);

      /* Jackknive over loci; that is, calculate a new estimate of Fs based on all
         alleles, except for one locus, and repeat by omitting a different locus
	 each time. 
      */
 
      JackFs = JackFsprim = JackFsSE = JackFsprimSE = 0.0; 
      resamples = n_loci;
      for (j = 0; j< resamples; ++j) {
	/* Calculate average Fs over all loci, eliminating the j'th locus each time  */
	ComputeF(1, n_loci, sample, sample + 1, &numerator, &denominator, 
		 &n_1, &n_2, &generations, &Kl, j );
	TEMP = numerator/denominator;
	JackFs += TEMP;            /* sum of JackFs             */
	JackFsSE += TEMP*TEMP;     /* sum of JackFs squared     */
	n_harmonic = 2.0/(1.0/n_1 + 1.0/n_2); 	
	/* using same equations as for the total data */
	if (plan == 2)
	  TEMP = (TEMP*(1.0 - 1.0/(4.0*n_harmonic)) - 1.0/n_harmonic);
	else
	  TEMP = (TEMP*(1.0 - 1.0/(4.0*n_harmonic) + 1.0/(4.0*N)) - 1.0/n_harmonic + 1.0/N);
	TEMP /= ((1.0 + TEMP/4.0)*(1-1.0/(2.0*n_2)));
	JackFsprim += TEMP;        /* sum of JackFsprim         */
	JackFsprimSE += TEMP*TEMP; /* sum of JackFsprim squared */
      } /* for */
      
      JackFsSE -= (JackFs*JackFs)/(double)(resamples);        /* sum of squares of JackFs */
      JackFsSE  = sqrt(JackFsSE*((resamples-1.0)/resamples));
      JackFs /= (double)resamples;                            /* mean of JackFs           */
      JackFsprimSE -= 
	(JackFsprim*JackFsprim)/(double)resamples; /* sum of squares of JackFsprim */
      JackFsprimSE = sqrt(JackFsprimSE*((resamples-1.0)/resamples));
      JackFsprim /= (double)resamples;                        /* mean of JackFsprim       */
      Ne = ComputeNe(JackFsprim, generations);    /* jackknife estimate of effective size */
      fprintf(outfile,"\nJackknife over %i loci (assumed independent):", resamples);
      fprintf(outfile,"\n                      Mean:       ");
      fprintf(outfile, "%12.7f %12.7f %6.1f  %8.0f", JackFs, JackFsprim, generations, Ne);
      fprintf(outfile,"\n                        SE:       ");
      fprintf(outfile, "%12.7f %12.7f", JackFsSE, JackFsprimSE);
      fprintf(outfile,"\n95%% Confidence intervals (Mean -/+ 1.96*SE):");
      fprintf(outfile,"\n       Lower ( 2.5%%) limit:       ");
      fprintf(outfile, "%12.7f %12.7f         %8.0f", 
	      JackFs-1.96*JackFsSE, JackFsprim -1.96*JackFsprimSE, 
	      ComputeNe(JackFsprim+1.96*JackFsprimSE, generations));
      fprintf(outfile,"\n       Upper (97.5%%) limit:       ");
      fprintf(outfile, "%12.7f %12.7f         %8.0f", 
	      JackFs+1.96*JackFsSE, JackFsprim +1.96*JackFsprimSE, 
	      ComputeNe(JackFsprim -1.96*JackFsprimSE, generations));
      
    } else  fprintf(outfile,"   fixed            "); /* nothing computed */
    fprintf(outfile,"\n\n");
  }  /* for sample */
  fprintf(outfile,"\nWhere\n");
  fprintf(outfile,"K = number of alleles\n");
  fprintf(outfile,"S1, S2 = number of genotyped individuals in each of the two samples\n"); 
  fprintf(outfile,"Fs = observed allele frequency shift over the sample interval\n");
  fprintf(outfile,"Fs' = d.o. corrected for expected contribution from sampling\n");
  fprintf(outfile,"t = number of generations between the two samples\n");
  fprintf(outfile,"Ne = estimated variance-effective population size per generation\n\n");
  if (outfile != stdout)
    fclose(outfile);
  printf("Result written to %s\n", outfilename);
  return 0;
}
